//-----------------------------------------------------------------------                  
//
//	FIFOtUARTʐM[
//
//-----------------------------------------------------------------------                  

#include "iodefine.h"
#include "r8c_uart.h"
#include "r8c_macro.h"


//-----------------------------------------------------------------------                  
//	`Ƃ
//-----------------------------------------------------------------------                  

#ifdef UART0
#define		stic		(*(volatile unsigned char *) (0x0051))		/* UART0 transmit interrupt control register */
#define		sric		(*(volatile unsigned char *) (0x0052))		/* UART0 receive interrupt control register */
#define		umr			(*(volatile unsigned char *) (0x00A0))		/* UART0 transmit/receive mode register */
#define		ubrg		(*(volatile unsigned char *) (0x00A1))		/* UART0 bit rate register */
#define		utb			(*(volatile union worddef *) (0x00A2))		/* UART0 transmit buffer register */
#define		uc0			(*(volatile unsigned char *) (0x00A4))		/* UART0 transmit/receive control register 0 */
#define		uc1			(*(volatile unsigned char *) (0x00A5))		/* UART0 transmit/receive control register 1 */
#define		urb			(*(volatile union uimr *) (0x00A6))			/* UART0 receive buffer register */
#endif

#ifdef UART1
#define		stic		(*(volatile unsigned char *) (0x0053))		/* UART0 transmit interrupt control register */
#define		sric		(*(volatile unsigned char *) (0x0054))		/* UART0 receive interrupt control register */
#define		umr			(*(volatile unsigned char *) (0x00A8))		/* UART0 transmit/receive mode register */
#define		ubrg		(*(volatile unsigned char *) (0x00A9))		/* UART0 bit rate register */
#define		utb			(*(volatile union worddef *) (0x00AA))		/* UART0 transmit buffer register */
#define		uc0			(*(volatile unsigned char *) (0x00AC))		/* UART0 transmit/receive control register 0 */
#define		uc1			(*(volatile unsigned char *) (0x00AD))		/* UART0 transmit/receive control register 1 */
#define		urb			(*(volatile union uirb *) (0x00AE))			/* UART0 receive buffer register */
#endif

#ifdef UART2
#define		stic		(*(volatile unsigned char *) (0x004B))		/* UART0 transmit interrupt control register */
#define		sric		(*(volatile unsigned char *) (0x004C))		/* UART0 receive interrupt control register */
#define		umr			(*(volatile unsigned char *) (0x0160))		/* UART0 transmit/receive mode register */
#define		ubrg		(*(volatile unsigned char *) (0x0161))		/* UART0 bit rate register */
#define		utb			(*(volatile union worddef *) (0x0162))		/* UART0 transmit buffer register */
#define		uc0			(*(volatile unsigned char *) (0x0164))		/* UART0 transmit/receive control register 0 */
#define		uc1			(*(volatile unsigned char *) (0x0165))		/* UART0 transmit/receive control register 1 */
#define		urb			(*(volatile union uirb *) (0x0166))			/* UART0 receive buffer register */
#endif


/*------------------------------------------------------
  UARTi receive buffer register
------------------------------------------------------*/
#define		rbl			urb.byte.low
#define		rbh			urb.byte.high
#define		tbl			utb.byte.low		/* UART0 transmit buffer register Low */
#define		tbh			utb.byte.high		/* UART0 transmit buffer register High */


//-----------------------------------------------------------------------                  
//	[΂ϐ
//-----------------------------------------------------------------------                  

#define		UART_ILVL	_BV(ILVL1)
typedef struct _fifo {
	SIZE_TYPE	idx_w;
	SIZE_TYPE	idx_r;
	SIZE_TYPE	count;
	char buff[ FIFO_SIZE ];
} FIFO;

volatile FIFO txfifo, rxfifo;


//-----------------------------------------------------------------------                  
//	UARTʐM\bh
//-----------------------------------------------------------------------                  

/* Initialize UART */

void uart_init()
{
	rxfifo.idx_r = 0;
	rxfifo.idx_w = 0;
	rxfifo.count = 0;
	txfifo.idx_r = 0;
	txfifo.idx_w = 0;
	txfifo.count = 0;

	umr = _BV(SMD2) | _BV(SMD0);		/* f[^Vrbg */
	ubrg = SYSCLK/BAUD/16-1;			/* {[[gݒ */
	stic = 0;							/* 荞݃xQ */
	sric = UART_ILVL;					/* 荞݃xQM荞ݗL*/
	uc0 = 0;
	uc1 = _BV(RE) | _BV(TE);			/* M */
#ifdef UART1
	u1pinsel = 1;						/* |[gTXDgp */
#endif
}


/* Get a received character */

SIZE_TYPE uart_test ()
{
	return rxfifo.count;
}


char uart_get ()
{
	char d;
	SIZE_TYPE i;


	i = rxfifo.idx_r;
	while(rxfifo.count == 0);
	d = rxfifo.buff[i++];
	DI();
	rxfifo.count--;
	EI();
	if(i >= FIFO_SIZE)
		i = 0;
	rxfifo.idx_r = i;

	return d;
}


/* Put a character to transmit */

void uart_put (char d)
{
	SIZE_TYPE i;


	i = txfifo.idx_w;
	while(txfifo.count >= FIFO_SIZE);
	DI();
	if ( stic & UART_ILVL ) {
		txfifo.buff[i++] = d;
		txfifo.count++;
	}
	else {
		tbl = d;
		stic = UART_ILVL;
	}
	EI();
	if(i >= FIFO_SIZE)
		i = 0;
	txfifo.idx_w = i;
}


/* UART RXC interrupt */
void UartReceive( void )
{
	union uirb d;
	SIZE_TYPE n, i;

	d = urb;
	if ( d.bit.sum ) return;
	n = rxfifo.count;
	if(n < FIFO_SIZE) {
		rxfifo.count = ++n;
		i = rxfifo.idx_w;
		rxfifo.buff[i++] = d.byte.low;
		if(i >= FIFO_SIZE)
			i = 0;
		rxfifo.idx_w = i;
	}
}


/* UART UDRE interrupt */
void UartTrance( void )
{
	SIZE_TYPE n, i;


	n = txfifo.count;
	if(n) {
		txfifo.count = --n;
		i = txfifo.idx_r;
		tbl = txfifo.buff[i++];
		if(i >= FIFO_SIZE)
			i = 0;
		txfifo.idx_r = i;
	}
	if(n == 0)
		stic = 0;
}

/* put strings to transmit */
void uart_puts( const char* pstr )
{
	while ( (*pstr)!='\0' ) {
		uart_put(*pstr);
		pstr++;
	}
}

void uart_write(const char* ptr, unsigned char size)
{
	while ( size-->0 ) {
		uart_put(*ptr);
		ptr++;
	}
}
